package com.gree.mall.miniapp.controller.user;

import com.alibaba.excel.util.CollectionUtils;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.miniapp.annotation.ApiNotAuth;
import com.gree.mall.miniapp.bean.Page;
import com.gree.mall.miniapp.bean.user.MyProfitBean;
import com.gree.mall.miniapp.bean.user.RankBean;
import com.gree.mall.miniapp.bean.user.RankItemBean;
import com.gree.mall.miniapp.bean.user.UserWxBean;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.helper.ResponseHelper;
import com.gree.mall.miniapp.logic.user.UserLogic;
import com.gree.mall.miniapp.plus.entity.AdminWebsit;
import com.gree.mall.miniapp.plus.entity.OrderShare;
import com.gree.mall.miniapp.plus.entity.User;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;

@Slf4j
@RestController
@Api(value = "小程序用户", tags ={"小程序用户"} )
@RequestMapping(value = "/user", produces = "application/json; charset=utf-8")
public class UserController {


    @Autowired
    UserLogic userLogic;



    @PostMapping("/bind")
    @ApiOperation(value = "绑定业务员")
    public ResponseHelper grantMobile(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId,
            @ApiParam(required = true, value = "业务员用户id") @RequestParam(required = true) String serviceId
    ) throws Exception {
        userLogic.bind(userId,serviceId);
        return ResponseHelper.success();
    }

    @GetMapping("/qrcode/bind")
    @ApiOperation(value = "获取业务员绑定小程序码")
    public ResponseHelper bindQrCode(HttpServletRequest request) throws Exception {
        return ResponseHelper.success(userLogic.bindQrCode(request));
    }
    @GetMapping("/rank")
    @ApiOperation(value = "上月业绩排行榜")
    public ResponseHelper rank(HttpServletRequest request) throws Exception {
        RankBean rank = userLogic.rank(request);
        if(rank != null && !CollectionUtils.isEmpty(rank.getRankList())) {
            for (RankItemBean bean : rank.getRankList()) {
                if (StringUtils.isNotEmpty(bean.getWorkUserName())) {
                    bean.setWorkUserName(bean.getWorkUserName().charAt(0) + "**");
                }
            }
        }
        return ResponseHelper.success(rank);
    }


    @ApiNotAuth
    @PostMapping("/auth")
    @ApiOperation(value = "微信授权")
    public ResponseHelper<UserWxBean> auth(
            @ApiParam(required = false,value="业务员用户id") @RequestParam(required = false) String serviceId,
            @ApiParam(required = true, value = "code") @RequestParam String code,
            @ApiParam(required = false, value = "授权类型true=企业微信 false= 微信，不传默认为微信") @RequestParam(required = false) Boolean work,
            @ApiParam(required = false, value = "商家编号") @RequestParam(required = false) String websitId,
            HttpServletRequest request
    ) throws RemoteServiceException, WxErrorException, InterruptedException {
        UserWxBean userWxBean = null;
        log.info("微信授权 code:{} serviceId:{} work:{}", code, serviceId, work);
        if(work == null || !work) {
            userWxBean = userLogic.addUser(code,serviceId,request);
        }
        userLogic.addVisit(request, userWxBean.getUserId());
        log.info(userWxBean.toString());
        return ResponseHelper.success(userWxBean);
    }

    @ApiNotAuth
    @PostMapping("/mobile/grant")
    @ApiOperation(value = "授权手机号")
    public ResponseHelper<UserWxBean> grantMobile(
            @ApiParam(required = true, value = "iv") @RequestParam(required = true) String iv,
            @ApiParam(required = false, value = "encryptedData") @RequestParam(required = false) String encryptedData,
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId,
            HttpServletRequest request
    ) throws Exception {
        UserWxBean wxMobile = userLogic.getWxMobile(iv, encryptedData, userId,request);
        return ResponseHelper.success(wxMobile);
    }

    @GetMapping("/user/detail")
    @ApiOperation(value = "个人信息")
    public ResponseHelper<UserWxBean> detail(
            @ApiParam(required = true, value = "code") @RequestParam(required = true) String userId,
            HttpServletRequest request
    ) throws RemoteServiceException {
        UserWxBean userWxBean = userLogic.userDetail(userId,request);
        return ResponseHelper.success(userWxBean);
    }

    @PostMapping("/userinfo/save")
    @ApiOperation(value = "保存微信账号的用户信息")
    public ResponseHelper<User> saveUserInfo(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId,
            @ApiParam(required = false, value = "用户昵称") @RequestParam(required = false) String nickName,
            @ApiParam(required = false, value = "性别( 未知 男 女)") @RequestParam(required = false) String sex,
            @ApiParam(required = false, value = "头像") @RequestParam(required = false) String avatarUrl,
            @ApiParam(required = false, value = "国家") @RequestParam(required = false) String country,
            @ApiParam(required = false, value = "省") @RequestParam(required = false) String province,
            @ApiParam(required = false, value = "城市") @RequestParam(required = false) String city
    ) throws Exception{
        User user = userLogic.saveUserInfo(userId, nickName, sex, avatarUrl, country, province, city);
        return ResponseHelper.success(user);
    }


    @GetMapping("/profit")
    @ApiOperation(value = "我的收益统计")
    public ResponseHelper<MyProfitBean> userProfit(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId
    ) throws RemoteServiceException {
        MyProfitBean myProfitBean = userLogic.myProfit(userId);
        return ResponseHelper.success(myProfitBean);
    }

    @GetMapping("/profit/list")
    @ApiOperation(value = "我的收益明细列表")
    public ResponseHelper<IPage<OrderShare>> userProfitList(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId,
            @ApiParam(required = false, value = "ING=结算中  OVER=已结算  CANCEL=已取消  EXCEPTION=异常结算") @RequestParam(required = false) String status,
            @ApiParam(value = "页号",required = true) @RequestParam Integer pageNum,
            @ApiParam(value = "页大小",required = true) @RequestParam Integer pageSize
    ) throws RemoteServiceException {
        IPage<OrderShare> myProfit = userLogic.myProfitList(userId,status,pageNum,pageSize);
        return ResponseHelper.success(myProfit);
    }




    @GetMapping("/profit/total")
    @ApiOperation(value = "总计收益")
    public ResponseHelper<BigDecimal> totalProfit(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId
    ) throws RemoteServiceException {
        BigDecimal bigDecimal = userLogic.totalProfit(userId);
        return ResponseHelper.success(bigDecimal);
    }

    @GetMapping("/customer")
    @ApiOperation(value = "我的客户列表")
    public ResponseHelper<Page<User>> customerList(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId,
            @ApiParam(required = true, value = "pageNum") @RequestParam(required = true) Integer pageNum,
            @ApiParam(required = true, value = "pageSize") @RequestParam(required = true)Integer pageSize
    ) throws RemoteServiceException {
        IPage<User> userIPage = userLogic.customerList(userId, pageNum, pageSize);
        return ResponseHelper.success(userIPage);
    }


    @GetMapping("/parent")
    @ApiOperation(value = "我的客户经理")
    public ResponseHelper<User> parentList(
            @ApiParam(required = true, value = "用户id") @RequestParam(required = true) String userId
    ) throws RemoteServiceException {
        User user = userLogic.parentUser(userId);
        return ResponseHelper.success(user);
    }



    @PostMapping("/userWebsit")
    @ApiOperation(value = "获得当前用户的门店")
    public ResponseHelper<AdminWebsit> userWebsit(
            @ApiParam(required = true, value = "经度") @RequestParam(required = true) String lng,
            @ApiParam(required = true, value = "纬度") @RequestParam(required = true) String lat,
            HttpServletRequest request
    ) throws RemoteServiceException {
        AdminWebsit adminWebsit = userLogic.userWebsit(lng,lat,request);
        return ResponseHelper.success(adminWebsit);
    }


    @PostMapping("/websitList")
    @ApiOperation(value = "获得所有商家")
    public ResponseHelper<List<AdminWebsit>> websitList(
            @ApiParam(required = true, value = "经度") @RequestParam(required = true) String lng,
            @ApiParam(required = true, value = "纬度") @RequestParam(required = true) String lat
    ) throws RemoteServiceException {
        List<AdminWebsit> adminWebsits = userLogic.websitList();
        return ResponseHelper.success(adminWebsits);
    }


    @PostMapping("/websitUrl")
    @ApiOperation(value = "返回营业执照")
    public ResponseHelper<String> websitUrl(
            HttpServletRequest request,
            @ApiParam(value = "网点id",required = true) @RequestParam String websitId,
            @ApiParam(value = "滑动验证key",required = false) @RequestParam(required = false) String key,
            @ApiParam(value = "滑动验证数值",required = false) @RequestParam(required = false) String vrifyCode
    ) throws Exception {
        String websitUrl = userLogic.websitUrl(websitId,key, vrifyCode, request);
        return ResponseHelper.success(websitUrl);
    }

}
