package com.gree.mall.miniapp.logic.user;

import com.aliyuncs.utils.StringUtils;
import com.gree.mall.miniapp.bean.UmsPayCallBean;
import com.gree.mall.miniapp.commonmapper.SoldNumMapper;
import com.gree.mall.miniapp.enums.OrderShareStatusEnum;
import com.gree.mall.miniapp.enums.OrderStatusEnum;
import com.gree.mall.miniapp.logic.acitivy.PromotionLuckDrawLogic;
import com.gree.mall.miniapp.logic.common.FucaiLogic;
import com.gree.mall.miniapp.logic.common.WechatLogic;
import com.gree.mall.miniapp.logic.order.OrderTaxLogic;
import com.gree.mall.miniapp.plus.entity.*;
import com.gree.mall.miniapp.plus.service.*;

import lombok.extern.slf4j.Slf4j;
import org.jdom.JDOMException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class PayLogic {

    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    OrderTaxLogic orderTaxLogic;
    @Autowired
    FucaiLogic fucaiLogic;
    @Autowired
    PromotionLuckDrawLogic promotionLuckDrawLogic;
    @Autowired
    UserService userService;
    @Autowired
    OrderInfoService orderInfoService;
    @Autowired
    OrderDetailService orderDetailService;
    @Autowired
    OrderShareService orderShareService;
    @Autowired
    UserCouponService userCouponService;

    @Autowired
    CouponGoodsService couponGoodsService;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private SoldNumMapper soldNumMapper;
    @Autowired
    private GoodsSpecService goodsSpecService;
    @Autowired
    PromotionGroupSpecService promotionGroupSpecService;
    @Autowired
    GiftExchangeCodeService giftExchangeCodeService;


    public void payCall(String id, String transactionId, BigDecimal totalFee, UmsPayCallBean bean) throws JDOMException, IOException {

        OrderInfo orderInfo = orderInfoService.getById(id);
        if (!orderInfo.getOrderStatus().equals(OrderStatusEnum.NOPAY.toString())) {
            log.info("请勿重复支付,orderId:{}", id);
            return ;
        }
        orderInfo.setOrderStatus(OrderStatusEnum.DFH.toString());
        orderInfo.setPayType(Objects.isNull(bean) ? "微信支付" : "云闪付");
        orderInfo.setPayTime(new Date());
        orderInfo.setTransactionId(transactionId);
        orderInfo.setDiscountAmount(orderInfo.getTotalAmount().add(orderInfo.getFreight()).subtract(orderInfo.getPayAmount()));
        if (Objects.nonNull(bean)) {
            orderInfo.setCouponMerchantContribute(new BigDecimal(bean.getCouponMerchantContribute()).divide(BigDecimal.valueOf(100)));
        }

        //计算各订单详情的优惠金额以及实际支付金额
        BigDecimal lastDiscountAmount = new BigDecimal(0);
        //实际支付金额
        BigDecimal payAmount = orderInfo.getPayAmount();

        //判断金额是否支付正确
        if(totalFee.doubleValue() != payAmount.doubleValue()){
            log.error("【支付回调严重bug】......orderId:{},回调金额:{},订单实际支付金额:{}",id,totalFee,payAmount);
            return;
        }

        payAmount = payAmount.subtract(orderInfo.getFreight());
        //商品总金额(不包含运费)
        BigDecimal totalProductAmount = orderInfo.getTotalProductAmount();
        //订单总商品数
        Integer totalNum = orderInfo.getTotalNum();
        List<OrderDetail> orderDetails = orderDetailService.lambdaQuery()
                .eq(OrderDetail::getOrderId, orderInfo.getOrderId())
                .orderByAsc(OrderDetail::getPrice)
                .list();

        for (OrderDetail orderDetail : orderDetails) {
            Goods goods = goodsService.getById(orderDetail.getGoodsId());
            Goods goodsUp = new Goods();
            goodsUp.setGoodsId(goods.getGoodsId());
            goodsUp.setSoldNum(goods.getSoldNum()+orderDetail.getNum());
            goodsUp.updateById();

        }



        //2021-10-29 商品券也根据满减平摊的方式处理，故注释以下代码
        //使用商品券的订单只对符合商品券使用的商品处理实际支付金额和优惠了多少金额
//        if(StringUtils.isNotEmpty(orderInfo.getUserCouponId())) {
//            UserCoupon userCoupon = userCouponService.getById(orderInfo.getUserCouponId());
//            CouponGoods couponGoods = couponGoodsService.lambdaQuery().eq(CouponGoods::getCouponId, userCoupon.getCouponId()).one();
//            if(userCoupon.getCouponType().equals(CouponTypeEnum.GOODS.toString())){
//                //如果是商品券，则将商品券的优惠金额+到实际支付金额，防止计算平摊的优惠金额出错
//                payAmount = payAmount.add(userCoupon.getCouponValue());
//                //记录商品券的优惠金额
//                for(OrderDetail orderDetail : orderDetails){
//                    if(orderDetail.getGoodsId().equals(couponGoods.getGoodsId())){
//                        orderDetail.setDiscountAmount(userCoupon.getCouponValue());
//                        orderDetail.setPayAmount(orderDetail.getTotalPrice().subtract(orderDetail.getDiscountAmount()));
//                    }
//                }
//            }
//        }

        //总的折扣优惠金额
        BigDecimal totalZkAmount = orderInfo.getPromotionDiscountAmount();
        //总的申请优惠金额
        BigDecimal totalApplyAmount = orderInfo.getPromotionApplyAmount();

        //订单优惠总金额(不包含商品券的优惠金额)
        BigDecimal orderDiscountAmount = totalProductAmount.subtract(payAmount);

        if(totalProductAmount.doubleValue() > 0) {
            BigDecimal orderDiscountPercent = orderDiscountAmount.divide(totalProductAmount, 4, BigDecimal.ROUND_DOWN);
            //公共的处理(可能涉及到修改了支付金额的)
            for (int i = 0; i < orderDetails.size(); i++) {
                OrderDetail orderDetail = orderDetails.get(i);
                if (Objects.nonNull(bean)) {
                    orderDetail.setCouponMerchantContribute(orderInfo.getCouponMerchantContribute());
                }
                updateSaleNum(orderDetail);
                //单价
                BigDecimal price = orderDetail.getNeedPayAmount() != null &&
                        orderDetail.getNeedPayAmount().compareTo(new BigDecimal("0.00")) > 0 ?
                        orderDetail.getNeedPayAmount() : orderDetail.getPrice();
                //当前商品所有优惠的优惠金额
                BigDecimal discount = orderDiscountPercent.multiply(price.multiply(BigDecimal.valueOf(orderDetail.getNum())))
                        .setScale(2, BigDecimal.ROUND_DOWN);
                //当前商品的实际支付金额比例
                BigDecimal percent = price.divide(orderDetail.getTotalPrice(), 4, BigDecimal.ROUND_DOWN);

                //当前商品折扣金额的优惠总金额
                BigDecimal singleZkAmount = totalZkAmount.multiply(percent);
                BigDecimal singleApplyAmount = totalApplyAmount.multiply(percent);

                if (i == orderDetails.size() - 1) {
                    //最后一个，直接拿总的减
                    BigDecimal subtract = orderDiscountAmount.subtract(lastDiscountAmount);
                    orderDetail.setDiscountAmount(orderDetail.getDiscountAmount().add(subtract));
                    orderDetail.setPayAmount(orderDetail.getPayAmount().subtract(orderDetail.getDiscountAmount()));

                    orderDetail.setZkAmount(totalZkAmount.subtract(singleZkAmount.multiply(BigDecimal.valueOf(i))));
                    orderDetail.setApplyAmount(totalApplyAmount.subtract(singleApplyAmount.multiply(BigDecimal.valueOf(i))));
                    continue;
                }

                //产品总金额
                BigDecimal amount = orderDetail.getPayAmount();
                //优惠后的实际支付金额
                BigDecimal truePrice = amount.subtract(discount);
                orderDetail.setPayAmount(truePrice);
                orderDetail.setDiscountAmount(orderDetail.getDiscountAmount().add(discount));
                orderDetail.setZkAmount(singleZkAmount);
                orderDetail.setApplyAmount(singleApplyAmount);
                lastDiscountAmount = lastDiscountAmount.add(discount);
            }
            orderDetailService.updateBatchById(orderDetails);
        }

        List<OrderDetail> promotionOrderDetails = orderDetails.stream().filter(v -> StringUtils.isNotEmpty(v.getPromotionGroupId())).collect(Collectors.toList());
        List<OrderDetail> commonlyOrderDetails = orderDetails.stream().filter(v -> StringUtils.isEmpty(v.getPromotionGroupId())).collect(Collectors.toList());
        //团购商品分佣金
        if(promotionOrderDetails.size() > 0)
            this.addOrderShare(orderInfo, promotionOrderDetails,2);
        //普通商品分佣金
        if(commonlyOrderDetails.size() > 0)
            this.addOrderShare(orderInfo, commonlyOrderDetails,1);

        //对接金蝶系统
       // Boolean aBoolean = fucaiLogic.sendSaleOrder(orderDetails,orderInfo,orderInfo.getWebsitNumber());
      //  orderInfo.setJindSaleStatus(aBoolean);

        orderInfo.updateById();

        //回写抽奖活动
        promotionLuckDrawLogic.updatePromotionLuckDrawCode(orderInfo);

        log.info("===============支付回调结束:{}==============", id);
    }

    /**
     * 修改销量
     */
    private void updateSaleNum(OrderDetail orderDetail) {
        try {
            //秒杀销售数量

            //团购活动销售数量
            List<PromotionGroupSpec> list = promotionGroupSpecService.lambdaQuery().eq(PromotionGroupSpec::getGoodsSpecId, orderDetail.getGoodsSpecId()).list();
            for(PromotionGroupSpec promotionGroupSpec : list){
                promotionGroupSpec.setSalesNum(promotionGroupSpec.getSalesNum() + orderDetail.getNum());
                promotionGroupSpec.updateById();
            }

            soldNumMapper.updateGoodsSpec(orderDetail);
            soldNumMapper.updateGoods(orderDetail);
        } catch (Exception e) {

        }
    }


    public String refundCall(String body) throws Exception {

        log.info("【微信退款回调开始】{}", body);
        Map<String, String> params = wechatLogic.doXMLParse(body);
        if (params.get("return_code").equals("SUCCESS")) {
            String reqInfo = params.get("req_info");
            Map map = wechatLogic.refundAESStr(reqInfo);
            String orderId = (String) map.get("out_trade_no");
            List<OrderDetail> list = orderDetailService.lambdaQuery()
                    .eq(OrderDetail::getOrderId, orderId)
                    .eq(OrderDetail::getRefund, true).list();
            BigDecimal totalShareAmount = new BigDecimal(0);
            for (OrderDetail orderDetail : list) {
                totalShareAmount = totalShareAmount.add(orderDetail.getShareAmount());
            }
            OrderShare orderShare = orderShareService.lambdaQuery().eq(OrderShare::getOrderId, orderId).one();
            if (orderShare != null) {
                orderShare.setAmount(orderShare.getAmount().subtract(totalShareAmount));
                orderShare.updateById();
            }
        }
        log.info("【微信退款回调结束】{}", body);
        return body;
    }



    /**
     * 新增结算记录
     * @param orderInfo
     * @param list
     * @param flag  1=分销员带货  2=团购佣金
     */
    public void addOrderShare(OrderInfo orderInfo, List<OrderDetail> list,int flag){
        try {
            BigDecimal shareAmount = list.stream().map(OrderDetail::getShareAmount).reduce(BigDecimal::add).orElse(BigDecimal.valueOf(0));
            if (shareAmount.doubleValue() < 0) {
                return;
            }

            String remark = "分销员带货";
            if(flag == 2){
                remark = "团购佣金";
            }

            OrderShare orderShare = new OrderShare();
            orderShare.setFlag(flag);
            orderShare.setPayType(orderInfo.getPayType());
            //分账
            if(flag == 1) {
                String workUserId = orderInfo.getWorkerId();
                if(StringUtils.isEmpty(workUserId)){
                    return;
                }
                List<User> workUserList = userService.lambdaQuery()
                        .eq(User::getWorkUserId, workUserId)
                        .eq(User::getCompanyWechatId,orderInfo.getCompanyWechatId())
                        .list();
                if(workUserList.size() == 0){
                    log.info("【结算团购】未找到分销员,分销员id:{}",workUserId);
                    return;
                }
                User workUser = workUserList.get(0);
                orderShare.setWorkUserId(workUserId);
                orderShare.setOpenId(workUser.getOpenId());
                orderShare.setWorkUserName(orderInfo.getWorkerName());
                orderShare.setWorkUserPhone(orderInfo.getWorkerPhone());
                orderShare.setWebsitName(orderInfo.getWebsitName());
                orderShare.setWebsitNumber(orderInfo.getWebsitNumber());
            }else if(flag == 2){
                orderShare.setPromotionGroupId(orderInfo.getPromotionGroupId());
                orderShare.setWorkUserId(orderInfo.getPromotionGroupUserId());
                User promotionGroupUser = userService.getById(orderInfo.getPromotionGroupUserId());
                if(promotionGroupUser == null){
                    log.info("【结算团购】未找到团长,团长id:{}",orderInfo.getPromotionGroupUserId());
                    return;
                }
                orderShare.setOpenId(promotionGroupUser.getOpenId());
                orderShare.setWorkUserName(promotionGroupUser.getNickName());
                orderShare.setWorkUserPhone(promotionGroupUser.getMobile());
            }

            BigDecimal totalShareAmount = list.stream().map(OrderDetail::getShareAmount)
                    .reduce(BigDecimal::add).orElse(new BigDecimal(0));
            //新增待分账记录
            orderShare.setOrderId(orderInfo.getOrderId());
            orderShare.setStatus(OrderShareStatusEnum.ING.toString());
            if (orderInfo.getPayType().equals("云闪付")) {
                // 云闪付线下结算
                orderShare.setStatus("OFFLINE");
            }
            orderShare.setAmount(totalShareAmount);
            orderShare.setUserId(orderInfo.getUserId());
            orderShare.setUserName(orderInfo.getUserName());
            orderShare.setPhone(orderInfo.getPhone());
            orderShare.setPayment(orderInfo.getPayAmount());
            orderShare.setOrderStatus(orderInfo.getOrderStatus());
            orderShare.setTransactionId(orderInfo.getTransactionId());
            orderShare.setPosition(orderInfo.getPosition());
            orderShare.setPayTime(orderInfo.getPayTime());
            StringBuffer address = new StringBuffer();
            address.append(orderInfo.getProvince())
                    .append(orderInfo.getCity())
                    .append(orderInfo.getArea())
                    .append(orderInfo.getStreet())
                    .append(orderInfo.getReceAddress()).append(orderInfo.getHouseNo());
            orderShare.setAddress(address.toString());
            orderShare.setRemark(remark);

            orderShare.setCreateTime(new Date());
            orderShare.setCompanyWechatId(orderInfo.getCompanyWechatId());
            orderShare.setCompanyWechatName(orderInfo.getCompanyWechatName());
            orderShare.insert();

            log.info("新增结算记录成功{}",orderInfo.getOrderId());
        }catch(Exception e){
            log.error("结算添加失败", e);
        }
    }



}
